@@ -2,37 +2,37 @@ import os
 import sys
 import subprocess
 import argparse
-import logging
 import json
 import psutil
 import time
 import six
-
+import signal
+import pdb
 
 from apscheduler.schedulers.background import BackgroundScheduler
 
-from . import fs_tracker
-from . import model
+from . import fs_tracker, model, logs
 from .local_queue import LocalQueue
 from .gpu_util import get_available_gpus, get_gpu_mapping, get_gpus_summary
 from .experiment import Experiment
-from .util import sixdecode, str2duration, retry
+from .util import sixdecode, str2duration, retry, LogReprinter
+from .model import parse_verbosity
 
-logging.basicConfig()
-logging.getLogger('apscheduler.scheduler').setLevel(logging.ERROR)
+logs.getLogger('apscheduler.scheduler').setLevel(logs.ERROR)
 
 
 class LocalExecutor(object):
-    """Runs job while capturing environment and logging results.
+    """Runs job while capturing environment and logs results.
     """
 
-    def __init__(self, args):
+    def __init__(self, queue, args):
         self.config = args.config
 
         if args.guest:
             self.config['database']['guest'] = True
 
-        self.logger = logging.getLogger('LocalExecutor')
+        self.task_queue = queue
+        self.logger = logs.getLogger('LocalExecutor')
         self.logger.setLevel(model.parse_verbosity(self.config.get('verbose')))
         self.logger.debug("Config: ")
         self.logger.debug(self.config)
@@ -57,7 +57,9 @@ class LocalExecutor(object):
                     if v is not None:
                         env[str(k)] = str(v)
 
-            fs_tracker.setup_experiment(env, experiment, clean=True)
+            env['PYTHONUNBUFFERED'] = 'TRUE'
+
+            fs_tracker.setup_experiment(env, experiment, clean=False)
             log_path = fs_tracker.get_artifact_cache('output', experiment.key)
 
             # log_path = os.path.join(model_dir, self.config['log']['name'])
@@ -70,9 +72,11 @@ class LocalExecutor(object):
 
             with open(log_path, 'w') as output_file:
                 python = 'python'
-                if experiment.pythonver == 3:
+                if experiment.pythonver[0] == '3':
                     python = 'python3'
 
+                python = which(python)
+
                 cmd = [python, experiment.filename] + experiment.args
                 cwd = experiment.artifacts['workspace']['local']
                 container_artifact = experiment.artifacts.get('_singularity')
@@ -89,7 +93,7 @@ class LocalExecutor(object):
                         if not art['mutable'] and os.path.exists(local_path):
                             os.symlink(
                                 art['local'],
-                                os.path.join(cwd, '..', tag)
+                                os.path.join(os.path.dirname(cwd), tag)
                             )
 
                     if experiment.filename is not None:
@@ -101,7 +105,7 @@ class LocalExecutor(object):
                     else:
                         cmd = ['singularity', 'run', container]
 
-                self.logger.info('Running cmd: \n {} '.format(cmd))
+                self.logger.info('Running cmd: {0} in {1}'.format(cmd, cwd))
 
                 p = subprocess.Popen(
                     cmd,
@@ -110,8 +114,15 @@ class LocalExecutor(object):
                     env=env,
                     cwd=cwd
                 )
-                # simple hack to show what's in the log file
-                ptail = subprocess.Popen(["tail", "-f", log_path])
+
+                run_log_reprinter = True
+                log_reprinter = LogReprinter(log_path)
+                if run_log_reprinter:
+                    log_reprinter.run()
+
+                def kill_subprocess():
+                    log_reprinter.stop()
+                    p.kill()
 
                 minutes = 0
                 if self.config.get('saveWorkspaceFrequency'):
@@ -147,32 +158,53 @@ class LocalExecutor(object):
                     minutes=minutes)
 
                 def kill_if_stopped():
-                    if db.get_experiment(
+                    try:
+                        db_expr = db.get_experiment(
                             experiment.key,
-                            getinfo=False).status == 'stopped':
-                        p.kill()
+                            getinfo=False)
+                    except:
+                        db_expr = None
+
+                    # Transient issues with getting experiment data might
+                    # result in a None value being returned, as result
+                    # leave the experiment running because we wont be able to
+                    # do anything else even if this experiment is stopped
+                    # in any event if the experiment runs too long then it
+                    # will exceed its allocated time and stop
+                    if db_expr is not None:
+                        if db_expr.status == 'stopped':
+                            kill_subprocess()
+                            return
 
                     if experiment.max_duration is not None and \
                             time.time() > experiment.time_started + \
                             int(str2duration(experiment.max_duration)
                                 .total_seconds()):
 
-                        p.kill()
+                        kill_subprocess()
+                        return
+
+                    # If our tasks queue is signalled inactive
+                    # during work process execution, that means we need to drop
+                    # current execution and exit
+                    if not self.task_queue.is_active():
+                        kill_subprocess()
 
                 sched.add_job(kill_if_stopped, 'interval', seconds=10)
 
                 try:
                     p.wait()
                 finally:
+                    log_reprinter.stop()
                     save_metrics(metrics_path)
                     sched.shutdown()
-                    ptail.kill()
                     db.checkpoint_experiment(experiment)
                     db.finish_experiment(experiment)
+                    return p.returncode
 
 
 def allocate_resources(experiment, config=None, verbose=10):
-    logger = logging.getLogger('allocate_resources')
+    logger = logs.getLogger('allocate_resources')
     logger.setLevel(verbose)
     logger.info('Allocating resources {} for experiment {}'
                 .format(experiment.resources_needed, experiment.key))
@@ -182,23 +214,31 @@ def allocate_resources(experiment, config=None, verbose=10):
         if experiment.resources_needed else 0
 
     if gpus_needed > 0:
-        ret_val = ret_val and allocate_gpus(gpus_needed, config)
+        ret_val = ret_val and allocate_gpus(gpus_needed,
+                                            experiment.resources_needed,
+                                            config)
     else:
-        allocate_gpus(0, config)
+        allocate_gpus(0)
 
     return ret_val
 
 
-def allocate_gpus(gpus_needed, config=None):
-    if gpus_needed <= 0:
+def allocate_gpus(gpus_needed, resources_needed={}, config=None):
+    # Only disable gpus if gpus_needed < 0
+    if gpus_needed < 0:
         os.environ['CUDA_VISIBLE_DEVICES'] = '-1'
         return True
+    elif gpus_needed == 0:
+        return True
 
-    available_gpus = get_available_gpus()
+    gpu_mem_needed = resources_needed.get('gpuMem', None)
+    strict = resources_needed.get('gpuMemStrict', False)
+
+    available_gpus = get_available_gpus(gpu_mem_needed, strict)
     gpu_mapping = get_gpu_mapping()
-    mapped_gpus = [str(gpu_mapping[g])
-                   for g in available_gpus]
+    mapped_gpus = [str(gpu_mapping[g]) for g in available_gpus]
 
+    os.environ['CUDA_DEVICE_ORDER'] = 'PCI_BUS_ID'
     if len(mapped_gpus) >= gpus_needed:
         os.environ['CUDA_VISIBLE_DEVICES'] = ','.join(
             mapped_gpus[:gpus_needed])
@@ -221,13 +261,41 @@ def main(args=sys.argv):
     parser.add_argument(
         '--timeout',
         default=0, type=int)
+    parser.add_argument(
+        '--verbose',
+        default='error')
+
+    # Register signal handler for signal.SIGUSR1
+    # which will invoke built-in Python debugger:
+    signal.signal(signal.SIGUSR1, lambda sig, stack: pdb.set_trace())
 
     parsed_args, script_args = parser.parse_known_args(args)
+    verbose = parse_verbosity(parsed_args.verbose)
 
-    queue = LocalQueue()
+    queue = LocalQueue(verbose=verbose)
     # queue = glob.glob(fs_tracker.get_queue_directory() + "/*")
     # wait_for_messages(queue, parsed_args.timeout)
-    worker_loop(queue, parsed_args, timeout=parsed_args.timeout)
+    returncode = worker_loop(queue, parsed_args, timeout=parsed_args.timeout)
+    sys.exit(returncode)
+
+
+def which(program):
+    import os
+
+    def is_exe(fpath):
+        return os.path.isfile(fpath) and os.access(fpath, os.X_OK)
+
+    fpath, fname = os.path.split(program)
+    if fpath:
+        if is_exe(program):
+            return program
+    else:
+        for path in os.environ["PATH"].split(os.pathsep):
+            exe_file = os.path.join(path, program)
+            if is_exe(exe_file):
+                return exe_file
+
+    return None
 
 
 def worker_loop(queue, parsed_args,
@@ -237,15 +305,15 @@ def worker_loop(queue, parsed_args,
 
     fetch_artifacts = True
 
-    logger = logging.getLogger('worker_loop')
+    logger = logs.getLogger('worker_loop')
 
     hold_period = 4
+    retval = 0
     while True:
         msg = queue.dequeue(acknowledge=False, timeout=timeout)
         if not msg:
             break
 
-        # first_exp, ack_key = queue.dequeue(acknowledge=False)
         first_exp, ack_key = msg
 
         data_dict = json.loads(sixdecode(first_exp))
@@ -260,10 +328,9 @@ def worker_loop(queue, parsed_args,
 
         logger.setLevel(verbose)
 
-        logger.debug('Received experiment {} with config {} from the queue'.
-                     format(experiment_key, config))
+        logger.debug('Received message: \n{}'.format(data_dict))
 
-        executor = LocalExecutor(parsed_args)
+        executor = LocalExecutor(queue, parsed_args)
 
         with model.get_db_provider(config) as db:
             # experiment = experiment_from_dict(data_dict['experiment'])
@@ -283,7 +350,7 @@ def worker_loop(queue, parsed_args,
                 int(str2duration(config['experimentLifetime'])
                     .total_seconds()) + experiment.time_added < time.time():
                 logger.info(
-                    'Experiment expired (max lifetime of {} was exceeded)'
+                    'Experiment expired (max lifetime of {0} was exceeded)'
                     .format(config.get('experimentLifetime'))
                 )
                 queue.acknowledge(ack_key)
@@ -300,7 +367,7 @@ def worker_loop(queue, parsed_args,
 
                 try:
                     python = 'python'
-                    if experiment.pythonver == 3:
+                    if experiment.pythonver[0] == '3':
                         python = 'python3'
                     if '_singularity' not in experiment.artifacts.keys():
                         pip_diff = pip_needed_packages(
@@ -326,30 +393,39 @@ def worker_loop(queue, parsed_args,
                         if fetch_artifacts or 'local' not in art.keys():
                             logger.info('Fetching artifact ' + tag)
                             if tag == 'workspace':
-                                art['local'] = db.get_artifact(
-                                    art, only_newer=False)
+                                art['local'] = retry(lambda: db.get_artifact(
+                                    art,
+                                    only_newer=False),
+                                    sleep_time=10,
+                                    logger=logger)
                             else:
-                                art['local'] = db.get_artifact(art)
-                    executor.run(experiment)
+                                art['local'] = retry(
+                                    lambda: db.get_artifact(art),
+                                    sleep_time=10,
+                                    logger=logger
+                                )
+
+                    returncode = executor.run(experiment)
+                    if returncode != 0:
+                        retval = returncode
                 finally:
                     sched.shutdown()
                     queue.acknowledge(ack_key)
 
                 if single_experiment:
                     logger.info('single_experiment is True, quitting')
-                    return
+                    return retval
             else:
                 logger.info('Cannot run experiment ' + experiment.key +
                             ' due lack of resources. Will retry')
-                time.sleep(config['sleep_time'])
+                # Debounce failed requests we cannot service yet
+                time.sleep(config.get('sleep_time', 5))
 
-        # wait_for_messages(queue, timeout, logger)
-
-        # queue = glob.glob(fs_tracker.get_queue_directory() + "/*")
-
-    logger.info("Queue in {} is empty, quitting"
+    logger.info("Queue in {0} is empty, quitting"
                 .format(fs_tracker.get_queue_directory()))
 
+    return retval
+
 
 def pip_install_packages(packages, python='python', logger=None):
     pipp = subprocess.Popen(
