@@ -1,39 +1,39 @@
 import sys
 import argparse
-import logging
 import json
 import re
 import os
 import uuid
 import importlib
 import time
-import multiprocessing
 import six
 import traceback
-from contextlib import closing
 import numpy as np
 
+from datetime import timedelta
+
 from .local_queue import LocalQueue
 from .pubsub_queue import PubsubQueue
 from .sqs_queue import SQSQueue
+from .qclient_cache import get_cached_queue
 from .gcloud_worker import GCloudWorkerManager
 from .ec2cloud_worker import EC2WorkerManager
 from .hyperparameter import HyperparameterParser
 from .util import rand_string, Progbar, rsync_cp
 from .experiment import create_experiment
+from .unencrypted_payload_builder import UnencryptedPayloadBuilder
+from .encrypted_payload_builder import EncryptedPayloadBuilder
 
 from . import model
-from . import auth
 from . import git_util
 from . import local_worker
 from . import fs_tracker
+from . import logs
+from .auth import get_auth
 
 
-logging.basicConfig()
-
-
-def main(args=sys.argv):
-    logger = logging.getLogger('studio-runner')
+def main(args=sys.argv[1:]):
+    logger = logs.getLogger('studio-runner')
     parser = argparse.ArgumentParser(
         description='Studio runner. \
                      Usage: studio run <runner_arguments> \
@@ -127,7 +127,7 @@ def main(args=sys.argv):
         default=None)
 
     parser.add_argument(
-        '--metric', '-m',
+        '--metric',
         help='Metric to show in the summary of the experiment, ' +
              'and to base hyperparameter search on. ' +
              'Refers a scalar value in tensorboard log ' +
@@ -220,13 +220,21 @@ def main(args=sys.argv):
         default=None
     )
 
+    parser.add_argument(
+        '--port',
+        help='Ports to open on a cloud instance',
+        default=[], action='append'
+    )
+
     # detect which argument is the script filename
     # and attribute all arguments past that index as related to the script
-    py_suffix_args = [i for i, arg in enumerate(args) if arg.endswith('.py')]
+    (runner_args, other_args) = parser.parse_known_args(args)
+    py_suffix_args = [i for i, arg in enumerate(args)
+                      if arg.endswith('.py') or '::' in arg]
+
     rerun = False
     if len(py_suffix_args) < 1:
         print('None of the arugments end with .py')
-        (runner_args, other_args) = parser.parse_known_args(args[1:])
         if len(other_args) == 0:
             print("Trying to run a container job")
             assert runner_args.container is not None
@@ -242,7 +250,7 @@ def main(args=sys.argv):
     else:
         script_index = py_suffix_args[0]
         exec_filename, other_args = args[script_index], args[script_index + 1:]
-        runner_args = parser.parse_args(args[1:script_index])
+        runner_args = parser.parse_args(args[:script_index])
 
     # TODO: Queue the job based on arguments and only then execute.
 
@@ -427,7 +435,9 @@ def main(args=sys.argv):
 def add_experiment(args):
     try:
         config, python_pkg, e = args
+
         e.pythonenv = add_packages(e.pythonenv, python_pkg)
+
         with model.get_db_provider(config) as db:
             db.add_experiment(e)
     except BaseException:
@@ -441,14 +451,11 @@ def get_worker_manager(config, cloud=None, verbose=10):
         return None
 
     assert cloud in ['gcloud', 'gcspot', 'ec2', 'ec2spot']
-    logger = logging.getLogger('runner.get_worker_manager')
+    logger = logs.getLogger('runner.get_worker_manager')
     logger.setLevel(verbose)
 
-    auth_cookie = None if config['database'].get('guest') \
-        else os.path.join(
-        auth.TOKEN_DIR,
-        config['database']['apiKey']
-    )
+    auth = get_auth(config['database']['authentication'])
+    auth_cookie = auth.get_token_file() if auth else None
 
     branch = config['cloud'].get('branch')
 
@@ -473,7 +480,13 @@ def get_worker_manager(config, cloud=None, verbose=10):
     return worker_manager
 
 
-def get_queue(queue_name=None, cloud=None, verbose=10):
+def get_queue(
+        queue_name=None,
+        cloud=None,
+        config=None,
+        logger=None,
+        close_after=None,
+        verbose=10):
     if queue_name is None:
         if cloud in ['gcloud', 'gcspot']:
             queue_name = 'pubsub_' + str(uuid.uuid4())
@@ -485,6 +498,14 @@ def get_queue(queue_name=None, cloud=None, verbose=10):
     if queue_name.startswith('ec2') or \
        queue_name.startswith('sqs'):
         return SQSQueue(queue_name, verbose=verbose)
+    elif queue_name.startswith('rmq_'):
+        return get_cached_queue(
+            name=queue_name,
+            route='StudioML.' + queue_name,
+            config=config,
+            close_after=close_after,
+            logger=logger,
+            verbose=verbose)
     elif queue_name == 'local':
         return LocalQueue(verbose=verbose)
     else:
@@ -515,7 +536,8 @@ def spin_up_workers(
                 worker_manager.start_worker(
                     queue_name, resources_needed,
                     ssh_keypair=runner_args.ssh_keypair,
-                    timeout=runner_args.cloud_timeout)
+                    timeout=runner_args.cloud_timeout,
+                    ports=runner_args.port)
         else:
             assert runner_args.bid is not None
             if runner_args.num_workers:
@@ -532,7 +554,8 @@ def spin_up_workers(
                 start_workers=start_workers,
                 queue_upscaling=queue_upscaling,
                 ssh_keypair=runner_args.ssh_keypair,
-                timeout=runner_args.cloud_timeout)
+                timeout=runner_args.cloud_timeout,
+                ports=runner_args.port)
 
     elif queue_name == 'local':
         worker_args = ['studio-local-worker']
@@ -546,7 +569,8 @@ def spin_up_workers(
         # logger.info('worker args: {}'.format(worker_args))
         if not runner_args.num_workers or int(runner_args.num_workers) == 1:
             if 'STUDIOML_DUMMY_MODE' not in os.environ:
-                local_worker.main(worker_args)
+                retval = local_worker.main(worker_args)
+                sys.exit(retval)
         else:
             raise NotImplementedError("Multiple local workers are not " +
                                       "implemented yet")
@@ -563,36 +587,53 @@ def submit_experiments(
     num_experiments = len(experiments)
     verbose = model.parse_verbosity(config['verbose'])
 
+    payload_builder = UnencryptedPayloadBuilder("simple-payload")
+    # Are we using experiment payload encryption?
+    key_path = config.get('public_key_path')
+    if key_path is not None:
+        logger.info("Using RSA public key path: {0}".format(key_path))
+        payload_builder = \
+            EncryptedPayloadBuilder(
+                "cs-rsa-encryptor [{0}]".format(key_path), key_path)
+
     start_time = time.time()
-    n_workers = min(multiprocessing.cpu_count() * 2, num_experiments)
 
-    with closing(multiprocessing.Pool(n_workers, maxtasksperchild=20)) as p:
-        experiments = p.imap_unordered(add_experiment,
-                                       zip([config] * num_experiments,
-                                           [python_pkg] *
-                                           num_experiments,
-                                           experiments),
-                                       chunksize=1)
-        p.close()
-        p.join()
+    # Update Python environment info for our experiments:
+    for experiment in experiments:
+        experiment.pythonenv = add_packages(experiment.pythonenv, python_pkg)
+
+    # Now add them to experiments database:
+    try:
+        with model.get_db_provider(config) as db:
+            for experiment in experiments:
+                db.add_experiment(experiment)
+    except BaseException:
+        traceback.print_exc()
+        raise
 
-    '''
     experiments = [add_experiment(e) for e in
-                    zip([config] * num_experiments,
-                        [python_pkg] *
-                        num_experiments,
-                        experiments)]
-    '''
+                   zip([config] * num_experiments,
+                       [python_pkg] *
+                       num_experiments,
+                       experiments)]
 
-    logger.info("Added %s experiments in %s seconds" %
-                (num_experiments, int(time.time() - start_time)))
 
-    queue = get_queue(queue_name, cloud, verbose)
-    for e in experiments:
-        queue.enqueue(json.dumps({
-            'experiment': e.__dict__,
-            'config': config}))
+    queue = get_queue(
+        queue_name=queue_name,
+        cloud=cloud,
+        config=config,
+        close_after=timedelta(
+            minutes=2),
+        logger=logger,
+        verbose=verbose)
 
+    for experiment in experiments:
+        payload = payload_builder.construct(experiment, config, python_pkg)
+        queue.enqueue(json.dumps(payload))
+        logger.info("studio run: submitted experiment " + experiment.key)
+
+    logger.info("Added {0} experiment(s) in {1} seconds to queue {2}"
+                .format(num_experiments, int(time.time() - start_time), queue_name))
     return queue.get_name()
 
 
@@ -717,7 +758,8 @@ def parse_artifacts(art_list, mutable):
                 'artifacts specfied by url can only be immutable'
             retval[tag] = {
                 'url': path,
-                'mutable': False
+                'mutable': False,
+                'unpack': False
             }
         elif s3_schema.match(entry) or \
                 gcs_schema.match(entry) or \
@@ -728,12 +770,14 @@ def parse_artifacts(art_list, mutable):
                 'artifacts specfied by url can only be immutable'
             retval[tag] = {
                 'qualified': path,
-                'mutable': False
+                'mutable': False,
+                'unpack': False
             }
         else:
             retval[tag] = {
                 'local': os.path.expanduser(path),
-                'mutable': mutable
+                'mutable': mutable,
+                'unpack': True
             }
             if not mutable:
                 assert os.path.exists(retval[tag]['local'])
@@ -865,8 +909,17 @@ def add_hyperparam_experiments(
 
 
 def add_packages(list1, list2):
-    pkg_dict = {re.sub('==.+', '', pkg): pkg for pkg in list1 + list2}
-    return [pkg for _, pkg in six.iteritems(pkg_dict)]
+    # This function dedups the package names which I think could be
+    # functionally not desirable however rather than changing the behavior
+    # instead we will do the dedup in a stable manner that prevents
+    # package re-ordering
+    pkgs = {re.sub('==.+', '', pkg): pkg for pkg in list1 + list2}
+    merged = []
+    for k in list1 + list2:
+        v = pkgs.pop(re.sub('==.+', '', k), None)
+        if v is not None:
+            merged.append(v)
+    return merged
 
 
 if __name__ == "__main__":
