@@ -1,4 +1,3 @@
-import logging
 import calendar
 
 try:
@@ -12,8 +11,7 @@ except ImportError:
     boto3 = None
 
 from .tartifact_store import TartifactStore
-logging.basicConfig()
-
+from . import logs
 
 class S3ArtifactStore(TartifactStore):
     def __init__(self, config,
@@ -37,23 +35,22 @@ class S3ArtifactStore(TartifactStore):
 
         if self.bucket not in [b['Name'] for b in buckets['Buckets']]:
             self.client.create_bucket(
-                Bucket=self.bucket,
-                CreateBucketConfiguration={
-                    'LocationConstraint': self.client._client_config
-                    .region_name
-                }
+                Bucket=self.bucket
             )
-
         super(S3ArtifactStore, self).__init__(
             measure_timestamp_diff,
             compression=compression,
             verbose=verbose)
 
+        self.set_storage_client(self.client)
+
+
     def _upload_file(self, key, local_path):
         self.client.upload_file(local_path, self.bucket, key)
 
-    def _download_file(self, key, local_path):
-        self.client.download_file(self.bucket, key, local_path)
+    def _download_file(self, key, local_path, bucket=None):
+        bucket = bucket or self.bucket
+        self.client.download_file(bucket, key, local_path)
 
     def _delete_file(self, key):
         self.client.delete_object(Bucket=self.bucket, Key=key)
@@ -74,10 +71,10 @@ class S3ArtifactStore(TartifactStore):
             Key=key)
 
     def _get_file_timestamp(self, key):
-        obj = boto3.resource('s3').Object(self.bucket, key)
 
         try:
-            time_updated = obj.last_modified
+            obj = self.client.head_object(Bucket=self.bucket, Key=key)
+            time_updated = obj.get('LastModified', None)
         except BaseException:
             return None
 
